/* ------------------------------------------------------------ */
/*
HTTrack Website Copier, Offline Browser for Windows and Unix
Copyright (C) Xavier Roche and other contributors

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


Important notes:

- We hereby ask people using this source NOT to use it in purpose of grabbing
emails addresses, or collecting any other private information on persons.
This would disgrace our work, and spoil the many hours we spent on it.


Please visit our Website: http://www.httrack.com
*/


/* ------------------------------------------------------------ */
/* File: Basic definitions                                      */
/*       Used in .c files for basic (malloc() ..) definitions   */
/* Author: Xavier Roche                                         */
/* ------------------------------------------------------------ */

#ifndef HTS_BASICH
#define HTS_BASICH

#include "htsglobal.h"

// size_t et mode_t
#include <stdio.h>
#include <stdlib.h>

#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#if HAVE_DLFCN_H
#include <dlfcn.h>
#endif

#if HTS_WIN
#else
#include <fcntl.h>
#endif
#include <assert.h>

#undef min
#undef max
#define min(a,b) ((a)>(b)?(b):(a))
#define max(a,b) ((a)>(b)?(a):(b))

// teste galit de 2 chars, case insensitive
#define hichar(a) ((((a)>='a') && ((a)<='z')) ? ((a)-('a'-'A')) : (a))
#define streql(a,b) (hichar(a)==hichar(b))

// is this MIME an hypertext MIME (text/html), html/js-style or other script/text type?
#define HTS_HYPERTEXT_DEFAULT_MIME "text/html"
#define is_hypertext_mime(a) \
   ( (strfield2((a),"text/html")!=0)\
  || (strfield2((a),"application/x-javascript")!=0) \
  || (strfield2((a),"text/css")!=0) \
  || (strfield2((a),"image/svg+xml")!=0) \
  || (strfield2((a),"image/svg-xml")!=0) \
  /*|| (strfield2((a),"audio/x-pn-realaudio")!=0) */\
  )

#define may_be_hypertext_mime(a) \
   (\
     (strfield2((a),"audio/x-pn-realaudio")!=0) \
  )


// caractre maj
#define isUpperLetter(a) ( ((a) >= 'A') && ((a) <= 'Z') )

// conversion ventuelle / vers antislash
#if HTS_WIN
char* antislash(char* s);
#else
#define antislash(A) (A)
#endif


// functions
#ifdef _WIN32
#define DynamicGet(handle, sym) GetProcAddress(handle, sym)
#else
#define DynamicGet(handle, sym) dlsym(handle, sym)
#endif

// tracer malloc()
#ifndef HTS_TRACE_MALLOC
#define malloct(A)          malloc(A)
#define calloct(A,B)        calloc(A,B)
#define freet(A)            do { assert((A) != NULL); if ((A) != NULL) { free(A); (A) = NULL; } } while(0)
#define realloct(A,B)       realloc(A,B)
#define memcpybuff(A, B, N) memcpy(A, B, N)
#else
/* debug version */
#define malloct(A)    hts_malloc(A)
#define calloct(A,B)  hts_calloc(A,B)
#define freet(A)      do { hts_free(A); (A) = NULL; } while(0)
#define realloct(A,B) hts_realloc(A,B)
void  hts_freeall();
void* hts_malloc    (size_t);
void* hts_calloc(size_t,size_t);
void* hts_xmalloc(size_t,size_t);
void  hts_free      (void*);
void* hts_realloc   (void*,size_t);
mlink* hts_find(char* adr);
/* protected memcpy */
#define memcpybuff(A, B, N) do { \
  mlink* lnk = hts_find((void*)(A)); \
  if (lnk != NULL) { \
    assert(lnk != NULL); \
    assert( * ( (t_htsboundary*) ( ((char*) lnk->adr) - sizeof(htsboundary) ) ) == htsboundary ); \
    assert( * ( (t_htsboundary*) ( ((char*) lnk->adr) + lnk->len ) ) == htsboundary ); \
    assert( ( ((char*)(A)) + (N)) < (char*) (lnk->adr + lnk->len) ); \
  } \
  memcpy(A, B, N); \
} while(0)

#endif

/* protected strcat, strncat and strcpy - definitely useful */
#define strcatbuff(A, B) do { \
  unsigned int sz = (unsigned int) strlen(A); \
  unsigned int szf = (unsigned int) strlen(B); \
  if (sizeof(A) != sizeof(char*)) { \
    assert(sz + szf + 1 < sizeof(A)); \
    if (szf > 0) { \
      if (sz + szf + 1 < sizeof(A)) { \
        memcpy((A) + sz, (B), szf + 1); \
      } \
    } \
  } else if (szf > 0) { \
    memcpybuff((A) + sz, (B), szf + 1); \
  } \
} while(0)
#define strncatbuff(A, B, N) do { \
  unsigned int sz = (unsigned int) strlen(A); \
  unsigned int szf = (unsigned int) strlen(B); \
  if (szf > (unsigned int) (N)) szf = (unsigned int) (N); \
  if (sizeof(A) != sizeof(char*)) { \
    assert(sz + szf + 1 < sizeof(A)); \
    if (szf > 0) { \
      if (sz + szf + 1 < sizeof(A)) { \
        memcpy((A) + sz, (B), szf); \
        * ( (A) + sz + szf) = '\0'; \
      } \
    } \
  } else if (szf > 0) { \
    memcpybuff((A) + sz, (B), szf); \
    * ( (A) + sz + szf) = '\0'; \
  } \
} while(0)
#define strcpybuff(A, B) do { \
  unsigned int szf = (unsigned int) strlen(B); \
  if (sizeof(A) != sizeof(char*)) { \
    assert(szf + 1 < sizeof(A)); \
    if (szf > 0) { \
      if (szf + 1 < sizeof(A)) { \
        memcpy((A), (B), szf + 1); \
      } else { \
        * (A) = '\0'; \
      } \
    } else { \
      * (A) = '\0'; \
    } \
  } else { \
    memcpybuff((A), (B), szf + 1); \
  } \
} while(0)
#define strncpybuff(A, B, N) do { \
  unsigned int szf = (unsigned int) strlen(B); \
  if (szf > (unsigned int) (N)) szf = (unsigned int) (N); \
  if (sizeof(A) != sizeof(char*)) { \
    assert(szf + 1 < sizeof(A)); \
    if (szf > 0) { \
      if (szf + 1 < sizeof(A)) { \
        memcpy((A), (B), szf); \
      } \
    } \
  } else { \
    memcpybuff((A), (B), szf); \
  } \
} while(0)

#endif
